//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Draw_Library, Example7 - The Back Buffer
//
// Date: 21st June, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example creates a 'complex primary surface', then retrieves an
// interface to the 'BackBuffer'.
//
// In order to do this, before creating the TDx_Draw component 'PrimarySurface',
// the example establishes the correct settings for creating a 'complex' primary
// surface.
//
// Then, during the TDx_Draw::OnCreate() event, a surface representing the
// backbuffer is retrieved using TDx_Draw::GetAttachedSurface().
//
// When painting, the example paints directly onto the backbuffer using VCL-style
// code and uses TDx_Draw::Flip() to display the backbuffer content on the monitor.
//
// Lastly, it shows how to destroy the backbuffer / primary surface correctly.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
AnsiString IntToHexStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
AnsiString string = "0x";
int value = 0x10000000;
for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=800;
Height=600;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
NumModes=0;
ModesListBox->Items->Clear();
Dx_Draw1->EnumDisplayModes(0,NULL,NULL);

Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE );

// create a complex primary surface
PrimarySurfaceDesc->Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
PrimarySurfaceDesc->BackBufferCount = 1;
PrimarySurfaceDesc->SCaps = PrimaryCaps;
PrimaryCaps->Caps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
PrimarySurface->Create( PrimarySurfaceDesc, Dx_Draw1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
PrimarySurface->Destroy();
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_NORMAL );
if (Dx_Draw1->RestoreDisplayMode())
    {
    Top=0;
    Left=0;
    Width=800;
    Height=600;
    Invalidate();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1EnumDisplayModes(TObject *Sender,
      TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished)
{
int width = SurfaceDesc->Width;
int height = SurfaceDesc->Height;
int refresh_rate = SurfaceDesc->RefreshRate;
int bit_depth;

if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = SurfaceDesc->PixelFormat->RGBBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = SurfaceDesc->PixelFormat->YUVBitCount[0];

if (NumModes<MAX_MODES)
    {
    Widths[NumModes] = width;
    Heights[NumModes] = height;
    BitDepths[NumModes] = bit_depth;
    RefreshRates[NumModes] = refresh_rate;
    NumModes++;

    ModesListBox->Items->Add( IntToStr(width)+"x"+IntToStr(height)+"x"+IntToStr(bit_depth)+" @ "+IntToStr(refresh_rate)+"Hz" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::NextModeButtonClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    Msg("Selecting Next Available Mode");
    if (ModesListBox->ItemIndex==ModesListBox->Items->Count-1) ModesListBox->ItemIndex=0;
    else ModesListBox->ItemIndex = ModesListBox->ItemIndex + 1;
    ModesListBox->OnClick(this);
    }
else
    ModesListBox->ItemIndex = -1;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ModesListBoxClick(TObject *Sender)
{
if (ModesListBox->ItemIndex!=-1)
    {
    int index = ModesListBox->ItemIndex;
    AnsiString name = ModesListBox->Items->Strings[index];

    Msg("Mode Selected: '"+name+"'");
    CurrentModeLabel->Caption = name;
    }
else
    CurrentModeLabel->Caption = "-- default --";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceCreate(TObject *Sender)
{
Msg(">>> Primary Surface Created");

// get it's attached backbuffer
if (PrimarySurface->GetAttachedSurface( BackBufferCaps, BackBuffer ))
    // call the backbuffer::OnCreate() event manually.
    BackBuffer->OnCreate(this);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceDestroy(TObject *Sender)
{
Msg(">>> Primary Surface Destroy");
if (BackBuffer->Created) BackBuffer->Destroy();
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::PaintButtonClick(TObject *Sender)
{
// create a canvas to use for drawing directly onto the primary surface
TCanvas* canvas = new TCanvas();
if (canvas==NULL)
    {
    ShowMessage("Could not create VCL canvas");
    return;
    }

// error-proof vcl code
HDC hdc;
// note: paint onto the BackBuffer, not the PrimarySurface
if (BackBuffer->GetDC(&hdc))
    {
    try
        {
        canvas->Handle = hdc;

        // paint the background star image
        canvas->StretchDraw( canvas->ClipRect, Image1->Picture->Graphic );

        // paint some text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Arial";
        canvas->Font->Style = TFontStyles();
        canvas->Font->Size = 20;
        canvas->Font->Color = (TColor) RGB(0,0,0);
        canvas->TextOut( 2, 2, "This message has been painted directly onto the BackBuffer" );
        canvas->Font->Color = (TColor) RGB(128,255,128);
        canvas->TextOut( 0, 0, "This message has been painted directly onto the BackBuffer" );

        // and more text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Comic Sans MS";
        canvas->Font->Style = TFontStyles() << fsBold << fsItalic << fsUnderline << fsStrikeOut;
        canvas->Font->Size = 24;
        canvas->Font->Color = (TColor) RGB(0,0,0);
        canvas->TextOut( 102, 42, "Comic Sans MS: 20Pt, Bold, Italic, Underline and StrikeOut" );
        canvas->Font->Color = (TColor) RGB(255,128,255);
        canvas->TextOut( 100, 40, "Comic Sans MS: 20Pt, Bold, Italic, Underline and StrikeOut" );
        }
    catch (Exception &e)
        {
        }

    BackBuffer->ReleaseDC(hdc);
    }

delete canvas;

Msg("BackBuffer has been painted");
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::PrimarySurfaceSurfaceLost(TObject *Sender)
{
PrimarySurface->Restore();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ModesListBoxDblClick(TObject *Sender)
{
SelectModeButton->OnClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelectModeButtonClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    ShowMessage("Please Note:\n\
You may experience difficulty restoring your display mode.\n\
To return to the original display mode at any time, press ALT-R (Restore) or ALT-X (eXit)" );

    int index = ModesListBox->ItemIndex;
    PrimarySurface->Destroy();
    Dx_Draw1->SetDisplayMode( Widths[index], Heights[index], BitDepths[index], RefreshRates[index], 0 );
    PrimarySurface->Create( PrimarySurfaceDesc, Dx_Draw1 );

    RepaintButton->OnClick(this);
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::RepaintButtonClick(TObject *Sender)
{
    int old_top = Top;
    Top=Screen->Height+1;
    Invalidate();
    Top=old_top;
    Invalidate();
}
//---------------------------------------------------------------------------



void __fastcall TMainForm::BackBufferCreate(TObject *Sender)
{
Msg(">>> BackBuffer Created");
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferDestroy(TObject *Sender)
{
Msg(">>> BackBuffer Destroyed");
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FlipButtonClick(TObject *Sender)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

ShowMessage("Please Note:\n\
After the primary surface has been painted the contents of the current form\n\
will no longer be visible though it is still active. To repaint the form\n\
Press ALT-R (Restore) or to exit the application, press ALT-X (eXit)" );

PrimarySurface->Flip( BackBuffer, DDFLIP_DONOTWAIT );
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BitBtn2Click(TObject *Sender)
{
Dx_Draw1->Create(NULL);
}
//---------------------------------------------------------------------------

