//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Draw_Library, ExampleA - Offscreen Surfaces
//
// Date: 21st June, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to create an offscreen surface that can be
// pre-rendered and then bit-blitted using surface-to-surface techniques
// to increase overall painting speed.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\ExtCtrls.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDCaps.h"
#include "TDDPixelFormat.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
#include <Dialogs.hpp>
#include <Graphics.hpp>
#include "TDx_DrawPalette.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 +
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TPanel *Panel1;
    TPanel *Panel2;
    TPanel *Panel3;
    TPanel *Panel4;
    TPanel *Panel5;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label5;

    TMemo *InfoMemo;
    TMemo *StatusMemo;
    TBitBtn *ExitButton;
    TBitBtn *ClearButton;
    TDx_Draw *Dx_Draw1;
    TListBox *ModesListBox;
    TLabel *Label7;
    TLabel *CurrentModeLabel;
    TLabel *Label1;
    TDx_DrawSurface *PrimarySurface;
    TDDSurfaceDesc *PrimarySurfaceDesc;
    TDDSCaps *PrimaryCaps;
    TButton *Button1;
    TButton *Button4;
    TBitBtn *SelectModeButton;
    TBitBtn *NextModeButton;
    TBitBtn *BitBtn2;
    TDx_DrawSurface *BackBuffer;
    TDDSurfaceDesc *BackBufferDesc;
    TDDSCaps *BackBufferCaps;
    TDx_DrawSurface *OffscreenSurface;
    TDDSurfaceDesc *OffscreenDesc;
    TDDSCaps *OffscreenCaps;
    TBitBtn *BitBtn1;
    TLabel *WidthLabel;
    TLabel *HeightLabel;
    TOpenDialog *OpenDialog1;
    TDDPixelFormat *OffscreenPixelFormat;
    TLabel *BitDepthLabel;
    TLabel *Label4;
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearButtonClick(TObject *Sender);

    void __fastcall Dx_Draw1Create(TObject *Sender);
    void __fastcall Dx_Draw1Destroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall Dx_Draw1EnumDisplayModes(TObject *Sender,
          TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished);
    void __fastcall NextModeButtonClick(TObject *Sender);
    void __fastcall ModesListBoxClick(TObject *Sender);
    void __fastcall PrimarySurfaceCreate(TObject *Sender);
    void __fastcall PrimarySurfaceDestroy(TObject *Sender);
    void __fastcall PrimarySurfaceSurfaceLost(TObject *Sender);
    void __fastcall ModesListBoxDblClick(TObject *Sender);
    void __fastcall SelectModeButtonClick(TObject *Sender);
    void __fastcall Button4Click(TObject *Sender);
    void __fastcall BackBufferCreate(TObject *Sender);
    void __fastcall BackBufferDestroy(TObject *Sender);
    void __fastcall BitBtn2Click(TObject* Sender);
    void __fastcall Button1Click(TObject *Sender);
    
    void __fastcall OffscreenSurfaceCreate(TObject *Sender);
    void __fastcall OffscreenSurfaceDestroy(TObject *Sender);
    void __fastcall BitBtn1Click(TObject *Sender);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
    
    
    
    
    void __fastcall OffscreenSurfaceSurfaceLost(TObject *Sender);
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

    void __fastcall OnIdle(TObject *Sender, bool& Done);

    #define MAX_MODES 50

    int         Widths[MAX_MODES];
    int         Heights[MAX_MODES];
    int         BitDepths[MAX_MODES];
    int         RefreshRates[MAX_MODES];
    int         NumModes;

    int         CurrentPageNumber;
    TCanvas*    canvas;
};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif

