//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Draw_Library,
//       ExampleC - Painting onto Surfaces
//
// Date: 21st June, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to paint onto the backbuffer using (3) different
// methods :-
//
// (1) paint onto the surface using a VCL-style TCanvas
// (2) paint onto the surface using standard Win32 code (somewhat)
// (3) paint onto the surface directly using custom paint code
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_DrawPalette"
#pragma link "tddbltfx"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
AnsiString IntToHexStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
AnsiString string = "0x";
int value = 0x10000000;
for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Top=0;
Left=0;
Width=800;
Height=600;
CurrentPageNumber = 0;

canvas = new TCanvas();
if (canvas==NULL) ShowMessage("Could not create VCL canvas");

Dx_Draw1->Create(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
if (canvas!=NULL) delete canvas;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
ModesListBox->Items->Clear();
NumModes=0;
Dx_Draw1->EnumDisplayModes(0,NULL,NULL);
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE );

// create: 1 x Complex Primary Surface, with 1 x attached Back Buffer, in an auto-flip configuration
PrimaryCaps->Caps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
PrimaryDesc->SCaps = PrimaryCaps;
PrimaryDesc->Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
PrimaryDesc->BackBufferCount = 1;
PrimarySurface->Create( PrimaryDesc, Dx_Draw1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
PrimarySurface->Destroy();
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_NORMAL );
if (Dx_Draw1->RestoreDisplayMode())
    {
    Top=0;
    Left=0;
    Width=800;
    Height=600;
    Invalidate();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1EnumDisplayModes(TObject *Sender,
      TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished)
{
int width = SurfaceDesc->Width;
int height = SurfaceDesc->Height;
int refresh_rate = SurfaceDesc->RefreshRate;
int bit_depth;

if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = SurfaceDesc->PixelFormat->RGBBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = SurfaceDesc->PixelFormat->YUVBitCount[0];

if (NumModes<MAX_MODES)
    {
    Widths[NumModes] = width;
    Heights[NumModes] = height;
    BitDepths[NumModes] = bit_depth;
    RefreshRates[NumModes] = refresh_rate;
    NumModes++;

    ModesListBox->Items->Add( IntToStr(width)+"x"+IntToStr(height)+"x"+IntToStr(bit_depth)+" @ "+IntToStr(refresh_rate)+"Hz" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::ModesListBoxClick(TObject *Sender)
{
if (ModesListBox->ItemIndex!=-1)
    {
    int index = ModesListBox->ItemIndex;
    AnsiString name = ModesListBox->Items->Strings[index];

    Msg("Mode Selected: '"+name+"'");
    CurrentModeLabel->Caption = name;
    }
else
    CurrentModeLabel->Caption = "-- default --";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceCreate(TObject *Sender)
{
Msg(">>> Primary Surface Created");

// get it's attached backbuffer
if (PrimarySurface->GetAttachedSurface( BackBufferCaps, BackBuffer ))
    // call the backbuffer::OnCreate() event manually.
    BackBuffer->OnCreate(this);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceDestroy(TObject *Sender)
{
Msg(">>> Primary Surface Destroy");
if (BackBuffer->Created) BackBuffer->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceSurfaceLost(TObject *Sender)
{
Dx_Draw1->RestoreAllSurfaces();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ModesListBoxDblClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    ShowMessage("Please Note:\n\
You may experience difficulty restoring your display mode.\n\
To return to the original display mode at any time, press ALT-R (Restore) or ALT-X (eXit)" );

    PrimarySurface->Destroy();

    int index = ModesListBox->ItemIndex;
    Dx_Draw1->SetDisplayMode( Widths[index], Heights[index], BitDepths[index], RefreshRates[index], 0 );

    PrimarySurface->Create( PrimaryDesc, Dx_Draw1 );
    RepaintButtonClick(this);
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::RepaintButtonClick(TObject *Sender)
{
Application->OnIdle = NULL;
if (Dx_Draw1->Created) Dx_Draw1->FlipToGDISurface();

    int old_top = Top;
    Top=Screen->Height+1;
    Invalidate();
    Top=old_top;
    Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferCreate(TObject *Sender)
{
Msg(">>> BackBuffer Created");
BackBuffer->GetSurfaceDesc( BackBufferDesc );
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferDestroy(TObject *Sender)
{
Msg(">>> BackBuffer Destroyed");
}
// --------------------------------------------------------------------------

void __fastcall TMainForm::OnSurfaceLost(TObject *Sender)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SnapshotButtonClick(TObject *Sender)
{
try {
    if (PrimarySurface->Created)
        if (PrimarySurface->SaveToFile( Dx_Draw1, GetCurrentDir() + "\\ExampleC_Snapshot.BMP" ))
            Msg("Saved screen snapshot to file: "+GetCurrentDir() + "\\ExampleC_Snapshot.BMP" );
    }
catch (...)
    {
    Msg("Error: Could not save a snapshot to file: " + GetCurrentDir() + "\\ExampleC_Snapshot.BMP" )
    RepaintButtonClick(this);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PaintVCLClick(TObject *Sender)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

ShowMessage("Please Note:\n\
After the primary surface has been painted the contents of the current form\n\
will no longer be visible though it is still active. To repaint the form\n\
\n\
Press ALT-S (Snapshot) to save a snapshot of the screen to the image source directory\n\
Press ALT-R (Restore) or to exit the application, press ALT-X (eXit)" );

randomize();
CurrentPageNumber = 0;
Application->OnIdle = VCLPaintFrame;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::VCLPaintFrame(TObject *Sender, bool& Done)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// this method contains the actual paint code.
// it :-
// (1) paints the entire backbuffer by stretching the pre-rendered offscreen surface so that it fills the entire backbuffer
// (2) paints some text over the top of the newly painted backbuffer
// (3) flips the backbuffer to the primary surface, displaying the newly painted backbuffer.


// paint the entire backbuffer BLACK
DDBltFX1->FillColor = ColorToRGB( clBlack );
BackBuffer->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, DDBltFX1 );

// paint some text over the top
HDC hdc;
if (BackBuffer->GetDC(&hdc))
    {
    try
        {
        // make sure to reassign the canvas so it paints in the right spot
        canvas->Handle = hdc;

        int width = canvas->ClipRect.Right - canvas->ClipRect.Left;
        int height = canvas->ClipRect.Bottom - canvas->ClipRect.Top;

        // paint some text
        canvas->Brush->Style = bsClear;
        canvas->Pen->Style = psClear;
        canvas->Font->Name = "Arial";
        canvas->Font->Style = TFontStyles();
        canvas->Font->Size = 20;
        canvas->Font->Color = (TColor) RGB(32,32,32);
        canvas->TextOut( 0, 0, "width: "+IntToStr(width)+", height: "+IntToStr(height) );
        canvas->Font->Color = (TColor) RGB( (CurrentPageNumber+192) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+64) % 256 );
        canvas->TextOut( 2, 2, "width: "+IntToStr(width)+", height: "+IntToStr(height) );

        // and more text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Comic Sans MS";
        canvas->Font->Style = TFontStyles() << fsBold << fsItalic << fsUnderline << fsStrikeOut;
        canvas->Font->Size = 24;
        canvas->Font->Color = (TColor) RGB(32,32,32);
        canvas->TextOut( 0, 40, "Frame# "+IntToStr(CurrentPageNumber) );
        canvas->Font->Color = (TColor) RGB( (CurrentPageNumber+64) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+192) % 256 );
        canvas->TextOut( 2, 42, "Frame# "+IntToStr(CurrentPageNumber) );

        // paint some lines
        int x_center = width / 2;
        int y_center = height / 2;
        int x_offset = width / 4;
        int y_offset = height / 4;

        int total_steps = 100;

        int x_incr = width / total_steps;
        int y_incr = height / total_steps;

        canvas->Brush->Style = bsClear;
        canvas->Pen->Style = psSolid;
        canvas->Pen->Width = 1;

        int x1 = x_center - x_offset - (CurrentPageNumber % x_offset);
        int x2 = x_center + x_offset + (CurrentPageNumber % x_offset);
        int y1 = y_center - y_offset - (CurrentPageNumber % y_offset);
        int y2 = y_center + y_offset + (CurrentPageNumber % y_offset);

        for (int i=0;i<total_steps;i++)
            {
            canvas->Pen->Color = (TColor) RGB( (CurrentPageNumber+i+64) % 256, (CurrentPageNumber+i+128) % 256, (CurrentPageNumber+i+192) % 256 );

            canvas->MoveTo( x1, y1 );
            canvas->LineTo( x2, y2 );

            canvas->MoveTo( x1, y2);
            canvas->LineTo( x2, y1 );

            x1 += x_incr;
            x2 -= x_incr;
            y1 += y_incr;
            y2 -= y_incr;
            }

        // and remove our temporary canvas handle
        canvas->Handle = NULL;
        }
    catch (Exception &e)
        {
        Application->OnIdle = NULL;
        }

    BackBuffer->ReleaseDC(hdc);
    }

// display the result
PrimarySurface->Flip( BackBuffer, DDFLIP_WAIT );

// increment our animation counter
CurrentPageNumber++;

// continue animating asap
Done = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PaintWin32Click(TObject *Sender)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

ShowMessage("Please Note:\n\
After the primary surface has been painted the contents of the current form\n\
will no longer be visible though it is still active. To repaint the form\n\
\n\
Press ALT-S (Snapshot) to save a snapshot of the screen to the image source directory\n\
Press ALT-R (Restore) or to exit the application, press ALT-X (eXit)" );

randomize();
CurrentPageNumber = 0;
Application->OnIdle = Win32PaintFrame;
}
//---------------------------------------------------------------------------
#include "wingdi.h"
void __fastcall TMainForm::Win32PaintFrame(TObject *Sender, bool& Done)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// this method contains the actual paint code.
// it :-
// (1) paints the entire backbuffer by stretching the pre-rendered offscreen surface so that it fills the entire backbuffer
// (2) paints some text over the top of the newly painted backbuffer
// (3) flips the backbuffer to the primary surface, displaying the newly painted backbuffer.


// paint the entire backbuffer BLACK
DDBltFX1->FillColor = ColorToRGB( clBlack );
BackBuffer->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, DDBltFX1 );

// paint some text over the top
HDC hdc;
if (BackBuffer->GetDC(&hdc))
    {
    try {
        // This is my best attempt at using win32 code.
        // I do not know how to create and apply the various types of fonts with win32, so i didn't waste time on it

        int width = BackBufferDesc->Width;
        int height = BackBufferDesc->Height;

        bool pen_ok;
        bool brush_ok;
        bool font_ok;

        HPEN pen;
        HPEN oldpen;
        HBRUSH brush;
        HBRUSH oldbrush;
        HFONT font;
        HFONT oldfont;
        COLORREF temp_color = RGB(0, 0, 0);

        // paint some text
        if (pen = CreatePen(PS_NULL, 0, temp_color )) pen_ok = (oldpen = SelectObject(hdc, pen));
        if (brush = CreateSolidBrush( temp_color )) brush_ok = (oldbrush = SelectObject(hdc, brush));
        if (font = GetStockObject(ANSI_VAR_FONT)) font_ok = (oldfont = SelectObject(hdc, font));
        if (font_ok)
            {
            AnsiString string;

            string = "width: "+IntToStr(width)+", height: "+IntToStr(height);
            SetTextColor( hdc, RGB(32,32,32) );
            TextOut( hdc, 0, 0, string.c_str(), string.Length() );
            SetTextColor( hdc, RGB( (CurrentPageNumber+192) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+64) % 256 ) );
            TextOut( hdc, 2, 2, string.c_str(), string.Length() );

            string = "Frame# "+IntToStr(CurrentPageNumber);
            SetTextColor( hdc, RGB(0,0,0) );
            TextOut( hdc, 0, 40, string.c_str(), string.Length() );
            SetTextColor( hdc, RGB( (CurrentPageNumber+64) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+192) % 256 ) );
            TextOut( hdc, 2, 42, string.c_str(), string.Length() );
            }

        if (font_ok) { SelectObject( hdc, oldfont ); DeleteObject( font ); }
        if (brush_ok) { SelectObject( hdc, oldbrush ); DeleteObject( brush ); }
        if (pen_ok) { SelectObject( hdc, oldpen ); DeleteObject( pen ); }

        // paint some lines
        int x_center = width / 2;
        int y_center = height / 2;
        int x_offset = width / 4;
        int y_offset = height / 4;

        int total_steps = 100;
        int x_incr = width / total_steps;
        int y_incr = height / total_steps;

        POINT old_point;

        int x1 = x_center - x_offset - (CurrentPageNumber % x_offset);
        int x2 = x_center + x_offset + (CurrentPageNumber % x_offset);
        int y1 = y_center - y_offset - (CurrentPageNumber % y_offset);
        int y2 = y_center + y_offset + (CurrentPageNumber % y_offset);

        for (int i=0;i<total_steps;i++)
            {
            temp_color = RGB( (CurrentPageNumber+i+64) % 256, (CurrentPageNumber+i+128) % 256, (CurrentPageNumber+i+192) % 256 );

            if (pen = CreatePen( PS_SOLID, 1, temp_color )) pen_ok = (oldpen = SelectObject(hdc, pen));

            MoveToEx( hdc, x1, y1, &old_point );
            LineTo( hdc, x2, y2 );

            MoveToEx( hdc, x1, y2, &old_point );
            LineTo( hdc, x2, y1 );

            if (pen_ok) { SelectObject( hdc, oldpen ); DeleteObject( pen ); }

            x1 += x_incr;
            x2 -= x_incr;
            y1 += y_incr;
            y2 -= y_incr;
            }
        }
    catch (Exception &e)
        {
        Application->OnIdle = NULL;
        }

    BackBuffer->ReleaseDC(hdc);
    }

// display the result
PrimarySurface->Flip( BackBuffer, DDFLIP_WAIT );

// increment our animation counter
CurrentPageNumber++;

// continue animating asap
Done = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PaintCustomClick(TObject *Sender)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

ShowMessage("Please Note:\n\
After the primary surface has been painted the contents of the current form\n\
will no longer be visible though it is still active. To repaint the form\n\
\n\
Press ALT-S (Snapshot) to save a snapshot of the screen to the image source directory\n\
Press ALT-R (Restore) or to exit the application, press ALT-X (eXit)" );

randomize();
CurrentPageNumber = 0;
Application->OnIdle = CustomPaintFrame;
}
//---------------------------------------------------------------------------
bool    TMainForm::CustomPaintPixel( int x1, int y1, dword pixel_color )
{
// this PutPixel() code is :-
// (a) not entirely optimized
// (b) it has been 'padded out' to make it easier for people to read
// (c) it's better to 'inline' this code into all custom paint functions
//
// special notes:
//
// (a) this will make DirectX look more familiar to people who have
//     experienced the "old DOS days". It's basically the same as a
//     VESA 2.0+ Linear Frame Buffer, if you exclude (b) below...
//
// (b) for optimal performance, as has been done in the "old DOS days"
//     you would normally customize the code based on the physical size
//     of the data being stored. For example, it's quicker on a 32-bit
//     processer to process chunks of data in 32 bits. Using 8 bit data
//     slows down everything graphics related, and using 32 bits at a
//     time is optimal.
//
//     8-bit (store: 1 byte at a time),
//     16 bit (store: 2 bytes, or a word, at a time)
//     24 bit (store: 1 word + 1 byte at a time)
//     32 bit (store: 1 dword at a time)
//
//     For some reason, you cannot optimize like this for DirectX,
//     DirectX forces you to insert a 4-byte color value a byte at a time.

if (Buffer_Ptr==NULL) return false;
if (x1<0) return false;
if (y1<0) return false;
if (x1>=Buffer_Width) return false;
if (y1>=Buffer_Height) return false;

bool    ok;
dword   dest_addr     = (dword) &Buffer_Ptr[ y1 * Buffer_Pitch ];
switch (Buffer_Bits)
    {
    case 8:
        {
        byte* byte_ptr = (byte*) (dest_addr + x1);
        *byte_ptr = (byte) pixel_color;
        ok = true;
        }
        break;

    case 16:
        {
        byte* byte_ptr = (byte*) (dest_addr + x1*2);
        word r = (word) ( GetRValue(pixel_color) >> 3 );
        word g = (word) ( GetGValue(pixel_color) >> 3 );
        word b = (word) ( GetBValue(pixel_color) >> 3 );

        word color = (b << 10) + (r << 5) + g;
        *byte_ptr++ = (byte) ((color & 0xFF00) >> 8);
        *byte_ptr = (byte) color;
        ok = true;
        }
        break;

    case 24:
        {
        byte* byte_ptr = (byte*) (dest_addr + x1*3);
        *byte_ptr++ = (byte) GetBValue( pixel_color );
        *byte_ptr++ = (byte) GetGValue( pixel_color );
        *byte_ptr = (byte) GetRValue( pixel_color );
        ok = true;
        }
        break;

    case 32:
        {
        byte* byte_ptr = (byte*) (dest_addr + x1*4);
        *byte_ptr++ = (byte) GetBValue( pixel_color );
        *byte_ptr++ = (byte) GetGValue( pixel_color );
        *byte_ptr = (byte) GetRValue( pixel_color );
        ok = true;
        }
        break;

    default:

        ok = false;
        break;
    }
return ok;
}
//---------------------------------------------------------------------------
bool    TMainForm::CustomPaintLine( int x1, int y1, int x2, int y2, dword color )
{
// this code is a port from my old "Game Development Engine"
// it is a cool algorithm i read somewhere once upon a time.

bool    ok = true;
int     dx,dy,            // x & y differences
        x_inc,y_inc,    // pixels moved during draw
        error=0;        // the discriminant i.e. error i.e. decision variable

// compute deltas
dx = x2-x1;
dy = y2-y1;

// test which direction the line is going in i.e. slope angle
if (dx>=0) x_inc = 1;
else { x_inc = -1; dx = -dx; } // need absolute value for dx

// test y component of slope
if (dy>=0) y_inc = 1;
else { y_inc = -1; dy = -dy; } // need absolute value for dy

// if |slope| <= 1 we draw in the x-direction
if (dx>dy)
   {
   // draw the line
   for (int x=0;x<=dx;x++)
       {
       ok &= CustomPaintPixel( x1, y1, color );

       // adjust the discriminate
       error+=dy;

       // test if error overflowed
       if (error>dx)
          {
          error-=dx;

          // move to next line
          y1 += y_inc;
          }

       x1 += x_inc;
       }
   }
else
   {
   // if |slope| > 1 we draw in the y-direction
   for (int y=0;y<=dy;y++)
       {
       ok &= CustomPaintPixel( x1, y1, color );

       // adjust the discriminate
       error+=dx;

       // test if error overflowed
       if (error>dy)
          {
          error-=dy;

          // move to next pixel
          x1 += x_inc;
          }

       y1 += y_inc;
       }
   }

return ok;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CustomPaintFrame(TObject *Sender, bool& Done)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// DO NOT paint the entire backbuffer BLACK using DirectX

//DDBltFX1->FillColor = ColorToRGB( clBlack );
//BackBuffer->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, DDBltFX1 );

// make a place to store data for later display (don't have a custom font engine in this example)
AnsiString display1 = "";
AnsiString display2 = "";

// lock the backbuffer, and retrieve the data required to access the surface memory directly
if (BackBuffer->Lock( NULL, BackBufferDesc, DDLOCK_SURFACEMEMORYPTR ))
    {
    try
        {
        // store information about the buffer for later use in the custom rendering code
        Buffer_Width = BackBufferDesc->Width;
        Buffer_Height = BackBufferDesc->Height;
        Buffer_Pitch = BackBufferDesc->Pitch;

        // determine the current bit-depth setting of the backbuffer (same as the display)
        int bit_depth = 0;
        if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = BackBufferDesc->PixelFormat->AlphaBitDepth[0];
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = BackBufferDesc->PixelFormat->AlphaBitDepth[0];
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = BackBufferDesc->PixelFormat->AlphaBitDepth[0];
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = BackBufferDesc->PixelFormat->BumpBitCount[0];
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = BackBufferDesc->PixelFormat->BumpBitCount[0];
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = BackBufferDesc->PixelFormat->RGBBitCount[0];
        else if (BackBufferDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = BackBufferDesc->PixelFormat->YUVBitCount[0];

        Buffer_Bits = bit_depth;

        // this stores a pointer to physical RAM
        Buffer_Ptr = (char*) BackBufferDesc->Internal_DDSURFACEDESC2_Ptr->lpSurface;

        // save the display information for later use
        display1 = "[address:"+IntToHexStr((dword) Buffer_Ptr) + ", Bits:"+IntToStr(Buffer_Bits)+"]";
        display2 = "[Width:"+IntToStr(Buffer_Width)+", Height:"+IntToStr(Buffer_Height)+", Pitch:"+IntToStr(Buffer_Pitch)+"]";

        // use temporary variables
        int width = Buffer_Width;
        int height = Buffer_Height;

        // determine the color settings based on # of bits per pixel the display mode is using
        dword color = 0;
        switch (Buffer_Bits)
            {
            case 8: // (8) bits per pixel - 256 color table lookup
                {
                // convert the color from the data entry Edit Box
                try {
                    color = StrToInt( PaletteIndexEdit->Text );
                    }
                catch (Exception &e)
                    {
                    color = 1; // red to show "ERROR"
                    }
                }
                break;

            case 16: // (16) bits per pixel
            case 24: // (24) bits per pixel
            case 32: // (32) bits per pixel
                {
                // convert the color from the data entry Edit Boxes
                try
                    {
                    byte r = (byte) StrToInt( RedEdit->Text );
                    byte g = (byte) StrToInt( GreenEdit->Text );
                    byte b = (byte) StrToInt( BlueEdit->Text );

                    color = RGB(r,g,b);
                    }
                catch (Exception &e)
                    {
                    color = RGB(255,0,0); // red to show "ERROR"
                    }
                }
                break;
            }

        // fill entire buffer using a very slow method :- draw every single Pixel using a custom PutPixel() function.
        for (int y=0;y<height;y++)
            for (int x=0;x<width;x++)
                CustomPaintPixel( x, y, color );

        // paint some lines
        int num_steps = 100;
        int x_step = width / num_steps;
        int y_step = height / num_steps;

        int x1 = 0;
        int y1 = 0;
        int x2 = width-1;
        int y2 = height-1;

        for (int i=0;i<num_steps;i++)
            {
            dword temp_color = RGB( (CurrentPageNumber+i+64) % 256, (CurrentPageNumber+i+128) % 256, (CurrentPageNumber+i+192) % 256 );

            CustomPaintLine( x1, y1, x2, y2, temp_color );
            CustomPaintLine( x1, y2, x2, y1, temp_color );
            CustomPaintLine( x2, y1, x1, y2, temp_color );
            CustomPaintLine( x2, y2, x1, y1, temp_color );

            x1 += x_step;
            x2 -= x_step;
            }

        x1 = 0;
        x2 = width-1;
        for (int i=0;i<num_steps;i++)
            {
            dword temp_color = RGB( (CurrentPageNumber+i+64) % 256, (CurrentPageNumber+i+128) % 256, (CurrentPageNumber+i+192) % 256 );

            CustomPaintLine( x1, y1, x2, y2, temp_color );
            CustomPaintLine( x1, y2, x2, y1, temp_color );
            CustomPaintLine( x2, y1, x1, y2, temp_color );
            CustomPaintLine( x2, y2, x1, y1, temp_color );

            y1 += y_step;
            y2 -= y_step;
            }

        // paint a frame
        x1 = 0;
        y1 = 0;
        x2 = width-1;
        y2 = height-1;

        CustomPaintLine( x1, y1, x2, y1, color );
        CustomPaintLine( x2, y1, x2, y2, color );
        CustomPaintLine( x1, y2, x2, y2, color );
        CustomPaintLine( x1, y1, x1, y2, color );

        // done as a good example.
        }
    catch (Exception &e)
        {
        Application->OnIdle = NULL;
        }

    BackBuffer->Unlock( NULL );
    }

//---
// paint the saved text over the top using standard VCL style paint code
HDC hdc;
if (BackBuffer->GetDC(&hdc))
    {
    try
        {
        // make sure to reassign the canvas so it paints in the right spot
        canvas->Handle = hdc;

        int width = canvas->ClipRect.Right - canvas->ClipRect.Left;
        int height = canvas->ClipRect.Bottom - canvas->ClipRect.Top;

        // paint some text
        canvas->Brush->Style = bsClear;
        canvas->Pen->Style = psClear;
        canvas->Font->Name = "Arial";
        canvas->Font->Style = TFontStyles();
        canvas->Font->Size = 20;
        canvas->Font->Color = (TColor) RGB(32,32,32);
        canvas->TextOut( 0, 0, "width: "+IntToStr(width)+", height: "+IntToStr(height) );
        canvas->Font->Color = (TColor) RGB( (CurrentPageNumber+192) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+64) % 256 );
        canvas->TextOut( 2, 2, "width: "+IntToStr(width)+", height: "+IntToStr(height) );

        // and more text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Comic Sans MS";
        canvas->Font->Style = TFontStyles() << fsBold << fsItalic << fsUnderline;
        canvas->Font->Size = 24;
        canvas->Font->Color = (TColor) RGB(32,32,32);
        canvas->TextOut( 0, 40, "Frame# "+IntToStr(CurrentPageNumber) );
        canvas->Font->Color = (TColor) RGB( (CurrentPageNumber+64) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+192) % 256 );
        canvas->TextOut( 2, 42, "Frame# "+IntToStr(CurrentPageNumber) );

        // and more text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Comic Sans MS";
        canvas->Font->Style = TFontStyles() << fsBold << fsItalic << fsUnderline;
        canvas->Font->Size = 16;
        canvas->Font->Color = (TColor) RGB(32,32,32);
        canvas->TextOut( 0, 80, display1 );
        canvas->Font->Color = (TColor) RGB( (CurrentPageNumber+64) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+192) % 256 );
        canvas->TextOut( 2, 82, display1 );

        // and more text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Comic Sans MS";
        canvas->Font->Style = TFontStyles() << fsBold << fsItalic << fsUnderline;
        canvas->Font->Size = 16;
        canvas->Font->Color = (TColor) RGB(32,32,32);
        canvas->TextOut( 0, 100, display2 );
        canvas->Font->Color = (TColor) RGB( (CurrentPageNumber+64) % 256, (CurrentPageNumber+128) % 256, (CurrentPageNumber+192) % 256 );
        canvas->TextOut( 2, 102, display2 );

        // and remove our temporary canvas handle
        canvas->Handle = NULL;
        }
    catch (Exception &e)
        {
        Application->OnIdle = NULL;
        }

    BackBuffer->ReleaseDC(hdc);
    }

// display the result
PrimarySurface->Flip( BackBuffer, DDFLIP_WAIT );

// increment our animation counter
CurrentPageNumber++;

// continue animating asap
Done = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PaletteIndexScrollBarChange(TObject *Sender)
{
PaletteIndexEdit->Text = IntToStr( PaletteIndexScrollBar->Position );    
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RedScrollBarChange(TObject *Sender)
{
RedEdit->Text = IntToStr( RedScrollBar->Position );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GreenScrollBarChange(TObject *Sender)
{
GreenEdit->Text = IntToStr( GreenScrollBar->Position );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BlueScrollBarChange(TObject *Sender)
{
BlueEdit->Text = IntToStr( BlueScrollBar->Position );
}
//---------------------------------------------------------------------------
